#include <LiquidCrystal_I2C.h>
#include <SimpleDHT.h>
#include <Wire.h>

#define PIN_FAN 8 //pin control ventilator
#define PIN_HEAT 9 //pin control modul de incalzire
#define bPlus 3 //pin buton pentru temperatura referinta
#define bMinus 2 //pin buton pentru temperatura referinta

//declarare variabile
LiquidCrystal_I2C lcd(0x27, 16, 2);
SimpleDHT11 dht11(10);
short stareA, stareU;
byte temperature;
byte humidity;
volatile short dTmp;

void setup() {
  // configurare pini
  pinMode(PIN_FAN,OUTPUT); // pe OUTPUT scriu, dau write
  pinMode(PIN_HEAT,OUTPUT);
  pinMode(bPlus, INPUT); // dau read
  pinMode(bMinus, INPUT);
  
  stareA = 0; //starea actuala este 0 (nici ventilatorul nici incalzirea nu este pornita)
  stareU = 0; //indica schimbarea starii actuale in functie de datele primite de la senzor
  dTmp = 20;  //valoare default pentru temepratura de referinta

  //initializare lcd
  lcd.init();
  lcd.backlight();

  //initializare sistem de intreruperi pentru butoanele de setare a temperaturii de referinta
  attachInterrupt(digitalPinToInterrupt(bPlus), bPlusP, RISING);
  attachInterrupt(digitalPinToInterrupt(bMinus), bMinusP, RISING);

  //incepere comunicare seriala pentru a transmite temperatura actuala unei instante Microsoft Excel
  //si plotarea unui grafic in timp real
  Serial.begin(9600);
}

void loop() {

  //citire temperatura cu senzorul DHT11 
  int err = SimpleDHTErrSuccess;
  if ((err = dht11.read(&temperature, &humidity, NULL)) != SimpleDHTErrSuccess) {
    Serial.print("Read DHT11 failed, err="); Serial.println(err);delay(1000);
    return;
  }

  //afisare temperatura citita si temperatura referinta
  lcd.setCursor(0,0);
  lcd.print("A:"); lcd.print((int)temperature); lcd.print("*C "); 
  lcd.print("D:"); lcd.print(dTmp); lcd.print("*C");

  //trimiterea temperaturii citite prin comunicare seriala
  Serial.println((int)temperature);

  //determinarea starii urmatoare a sistemului pe baza temperaturii citite de la senzor, a starii actuale
  //si a temperaturii de referinta introdusa de utilizator
  if((int)temperature < dTmp - 1) // pentru ca exista fluctuatii
  {
    stareU = 1;
    lcd.setCursor(0,1);
    lcd.print("HEAT ON"); //afisare modul incalzire ON
  }
  else if(((int)temperature == dTmp - 1 && stareA == 2) || ((int)temperature == dTmp && stareA == 1)) // ambele oprite(stareA == 2 -> caldura, stareA == 1 -> racire)
  {
    stareU = 0;
    lcd.clear(); //nici ventilatorul nici incalzirea nu sunt pornite asa ca nu se va afisa nimic legat despre acestea
    lcd.setCursor(0,0);
    lcd.print("A:"); lcd.print((int)temperature); lcd.print("*C "); 
    lcd.print("D:"); lcd.print(dTmp); lcd.print("*C");
    
  }else if((int)temperature > dTmp + 1) // pentru ca exista fluctuatii
  {
    stareU = 2;
    lcd.setCursor(0,1);
    lcd.print("FAN ON"); //afisare ventilator ON
  }
  
  SwitchStates(stareA, stareU);
  stareA = stareU;
  delay(1500); //delay introdus pentru senzorul DHT11 care are nevoie de o secunda pentru a face citirea
}

//functie asociata sistemului de intreruperi
void bPlusP()
{
  if(dTmp < 32) //limitarea utilizatorului la introducerea unei temperaturi maxime de 32*C
  {
    dTmp = dTmp + 1;
  }
}

//functie asociata sistemului de intreruperi
void bMinusP()
{
  if(dTmp > 20) //limitarea utilizatorului la introducerea unei temperaturi minime de 20*C
  {
    dTmp = dTmp - 1;
  }
}

//functia care da comanda catre ventilator sau modulul de incalzire in functie de starea actuala si cea dorita
void SwitchStates(short stareActuala, short stareUrmatoare)
{
  if(stareActuala == 0) // niciunul nu e pornit
  {
    if(stareUrmatoare == 1) //heat on
    {
      digitalWrite(PIN_HEAT, HIGH); // aici o sa comute tranzistorul de caldura
    }else if(stareUrmatoare == 2) //fan on
    {
      digitalWrite(PIN_FAN, HIGH); // aici o sa comute tranzistorul de racire
    }
  }else if(stareActuala == 1)
  {
    if(stareUrmatoare == 0) //stare generala
    {
      digitalWrite(PIN_HEAT, LOW); // oprire caldura
    }else if(stareUrmatoare == 2)
    {
      digitalWrite(PIN_HEAT, LOW); // oprire caldura
      digitalWrite(PIN_FAN, HIGH); // pornire racire
    }
  }else if(stareActuala == 2)
  {
    if(stareUrmatoare == 0)
    {
      digitalWrite(PIN_FAN, LOW);
    }else if(stareUrmatoare == 1)
    {
      digitalWrite(PIN_FAN, LOW);
      digitalWrite(PIN_HEAT, HIGH);
    }
  }
}
